:- module header.

% reads in the file, looking for the :- infix header declaration.

:- interface.
:- import_module io, list, string.

:- pred read_header(list(string)::out, list(string)::out, 
                    state::di, state::uo) is det.

:- implementation.
:- import_module char, require, int, string_utils.

read_header(Fix_words, File) --> read_write_lines([], Fix_words, [], File).

% here we read in the file one line at a time and spit it out likewise.

:- pred read_write_lines(list(string)::in, list(string)::out, 
                         list(string)::in, list(string)::out, 
                         state::di, state::uo) is det.
read_write_lines(Words_in, Fix_words, FileIn, FileOut) -->
  read_line_as_string(Maybe_string),
  process_line(Maybe_string, Words_in, Fix_words, FileIn, FileOut).

:- pred process_line(result(string)::in, 
                     list(string)::in, list(string)::out,
                     list(string)::in, list(string)::out,
                     state::di, state::uo) is det.
process_line(error(Error), _, _, _, _) -->
  { error(format("Encountered error: %s", [s(error_message(Error))])) }.

process_line(eof, List, List, File, File) --> [].

process_line(ok(String), Words_in, Fix_words, FileIn, FOut) -->
  ({ found_infix(String, Words_in, Result) ->
     More_words = Result, Line = "";
     More_words = [], Line = String }),
  read_write_lines(Words_in ++ More_words, Fix_words, FileIn ++ [Line], FOut).

:- pred found_infix(string::in, list(string)::in, list(string)::out) is semidet.
found_infix(_Line, [H|T], [H|T]) :- fail.
found_infix(Line, [], map(chop, Tmp_words)) :-
  [":-", "infix" | Tmp_words] = words(is_whitespace, chop(Line)).
